/*
     spar - Show Process Accounting Records
     Copyright (C) 1993 Douglas Lee Schales, David K. Hess, David R. Safford

     Please see the file `COPYING' for the complete copyright notice.

config/template.h - 10/31/93

*/

/* This is a template file for describing the various settings 
   available in the config file.
*/

#ifndef __TEMPLATE_SPAR__
#define __TEMPLATE_SPAR__

/* 
  Is setlocale() available?  If so, process start times printed via
  the '@' format characters will be "localeized".
*/
#define USE_LOCALE

/*
   Where is the default pacct file?  This is usually /usr/adm/pacct
*/
#define DEFAULT_PACCT "/usr/adm/pacct"

/*
  How many characters of the command name are
  stored in the accounting file.
*/
#define AC_COMMLEN 8

/*
   Directory containing System V style pty names.
   i.e. /dev/pts/0, /dev/pts/1

   If undefined, no SYSVPTY lookups will be performed.
*/
#define SYSVPTY "/dev/pts"
/*
  Name of the System V MASTER pseudo tty
*/
#define SYSVMPTYC "/dev/ptmajor"
/*
  Name of the System V master side of the pseudo tty
  This will result in /dev/ptmx/0 style names, even though
  /dev/ptmx is not a directory.
*/
#define SYSVMPTY "/dev/ptmx"
/*
  Name of the System V directory containing real tty files
  (/dev/tty/0, /dev/tty/1)
*/
#define SYSVTTY "/dev/term"

/*
  Directory containing BSD style pty names (/dev/pty0a, etc).
*/
#define BSDPTY "/dev"
/*
  Directory containing BSD style tty names (/dev/tty01, etc)
#define BSDTTY "/dev"

/*
  A comma separated, (char *)0 terminated list of other device
  files that may be used as controlling terminals.
*/

#define MISCTTY "/dev/wscons", (char *)0

/*
  Header files needed for definitions below 
*/
#include <string.h>
#include <malloc.h>
#include <memory.h>
#include <stdlib.h>
#include <unistd.h>
#include <sys/acct.h>
#include <sys/sysmacros.h>

/*
  How to get the major dev from a 'dev_t' value.
*/
#define MAJORDEV(x) getemajor((x))
/*
  How to get the minor dev from a 'dev_t' value.
#define MINORDEV(x) geteminor((x))

/*
  The clock rate of the system.  A system call should be
  used if available if the binary is going to be used on
  different hardware.  The result is stored in the variable
  'clockrate' to prevent repeated system calls.
*/
#define CLKRATE (unsigned long)sysconf(_SC_CLK_TCK)

/*
  How to convert a comp_t data item to an integer value
  (probably the same on most 32 bit systems)
*/
#define fromcompt(t) (((t)&0x1fff)<<(3*(((t)&0xe000)>>13)))

/*
  How to convert from 'ticks' to seconds.  The external variable
  'clockrate' will have the system clockrate defined.

  In this case, ticks is stored in a comp_t data item.
*/
#define fromticks(t) ((fromcompt(t))/clockrate)

/*
  How to extract the exit signal number from the process exit status 
*/
#define ACSIGEXIT(n) ((n)&127)

/*
  How many bytes per click?  A system call should be used
  if available if the binary is going to be used on different
  hardware.
*/
#define BYTESPERCLICK (unsigned long)sysconf(_SC_PAGESIZE)

/*
  Additional 'print' format characters.  The routines to perform
  the "output" should be declared as shown.  The first operand is
  a 'struct acct *' which points to the current accounting record.
  The second operand is a user defined data item from the format
  definition table.
*/
extern char *fmtmemusage(struct acct *, int);
extern char *fmtchar(struct acct *, int);

/*
  The first element is the format character.  The extra formats
  are inserted ahead of the standard format characters, so any
  of the standard format characters can be overridden here.

  The second element is the default printf() formatting to use
  if the user doesn't specify one.  For varying width items,
  a width specifier should be specified (see format.h).

  The third element is the routine to handle the format character.
  This routine is expected to format its data and return a char *
  pointer to the result.  It should not write the data out itself.

  The fourth element is optional data passed to the routine as the
  second argument.  This allows the same routine to be used for
  handling different format characters.

  NOTE:  A ',' must follow the last entry.  Also, since this is a
  C preprocessor macro, new lines must be escaped as shown.
*/

#define EXTRA_FORMATS \
     { 'M', "%s", fmtmemusage, 0 },\
     { 'C', "%s", fmtchar, 0 },

#endif
