/*
     spar - Show Process Accounting Records
     Copyright (C) 1993 Douglas Lee Schales, David K. Hess, David R. Safford

     Please see the file `COPYING' for the complete copyright notice.

spar.c - 10/28/93

*/
#include <stdio.h>
#include <sys/types.h>
#include <sys/time.h>
#include <netdb.h>
#include <errno.h>
#include <sys/acct.h>

#include "config.h"
#include "version.h"

#ifdef USE_LOCALE
#include <locale.h>
#endif

#include "spar.h"
#include "chario.h"
#include "parser.h"
#include "interp.h"
#include "timesub.h"

#ifndef DEFAULT_FORMAT_STRING
#define DEFAULT_FORMAT_STRING "%t %F %c %u %d %U %S %E %0X\n"
#endif

static struct args defargs = {
     0,
     DEFAULT_FORMAT_STRING
};

extern char *optarg;
extern int optind;

extern struct parsetree *parse(void);
static void extract(int, struct parsetree *, int);
static void flushbuf(void);
static void writebin(struct acct *, struct args *);
static void writeascii(struct acct *, struct args *);
extern void writebuf(struct acct *, struct args *);
static void setlogmode(int);
static void setlogfile(FILE *);
extern char *gettimestr(struct timeval);
static void helpscreen(char *);
extern void writeformatted(FILE *, struct acct *, struct args *);
extern void flushoutput(FILE *);

extern char *cgetnametty(dev_t);
extern char *cgetnamuid(uid_t);

#define LOGBINARY 0
#define LOGASCII 1

static int unbuffered = 0;
int quietflag;

#ifndef DEFAULT_PACCT
#define DEFAULT_PACCT "/usr/adm/pacct"
#endif

static int outlogmode = LOGBINARY;
static FILE *logfile = stdout;

static void
setlogfile(FILE *f)
{
     logfile = f;
}

static void
setlogmode(int logmode)
{
     outlogmode = logmode;
}

extern char **environ;

int
main(int argc, char **argv)
{
     struct parsetree *parsetree;
     int logmode = LOGASCII;
     FILE *outfile = stdout;
     FILE *dumpfile = (FILE *)0;
     int file = 0, expr = 0, errflag = 0;
     int reverse = 0;
     int status;
     char *membufname;
     int c;
     char *sparformat = (char *)0, *s;

     quietflag = 0;

#ifdef USE_LOCALE
     setlocale(LC_TIME, "");
#endif

     if((s = getenv("SPARFORMAT"))){
	  sparformat = strdup(s);
     }
     else
	  sparformat = DEFAULT_FORMAT_STRING;

     while((c=getopt(argc, argv, "d:e:f:E:F:o:p:abhUrqv")) != -1){
	  switch(c){
	  case 'p':
	       sparformat = optarg;
	       break;
	  case 'v':
	       printf("spar, version %s\n", toolversion);
	       exit(0);
	       break;
	  case 'a':
	       logmode = LOGASCII;
	       break;
	  case 'b':
	       logmode = LOGBINARY;
	       break;
	  case 'q':
	       quietflag = 1;
	       break;
	  case 'r':
	       if(dumpfile==stdin){
		    (void)fprintf(stderr,
				  "Can not use reverse flag with stdin\n");
		    exit(1);
	       }
	       else
		    reverse = 1;
	       break;
	  case 'f':
	       if((status = addfile(optarg)) != NOERR){
		    errflag++;
		    if(status == FILEERR){
			 (void)perror(optarg);
			 exit(1);
		    }
		    else {
			 (void)fprintf(stderr,
				       "%s: unexpected error\n", optarg);
			 exit(1);
		    }
	       }
	       file++;
	       break;
	  case 'F':
	       if((status = includefile(optarg)) != NOERR){
		    errflag++;
		    if(status == FILEERR){
			 (void)perror(optarg);
			 exit(1);
		    }
		    else {
			 (void)fprintf(stderr,
				       "%s: unexpected error\n", optarg);
			 exit(1);
		    }
	       }
	       file++;
	       break;
	  case 'e':
	       membufname = (char *)malloc(20);
	       (void)sprintf(membufname, "argv[%d]", optind-1);
	       if((status = addmembuf(optarg, membufname)) != NOERR){
		    (void)fprintf(stderr, "-e: unexpected error\n");
		    errflag++;
	       }
	       expr++;
	       break;
	  case 'E':
	       membufname = (char *)malloc(20);
	       (void)sprintf(membufname, "argv[%d]", optind-1);
	       if((status = includemembuf(optarg, membufname)) != NOERR){
		    (void)fprintf(stderr, "-E: unexpected error\n");
		    errflag++;
	       }
	       expr++;
	       break;
	  case 'd':
	       if(strcmp(optarg, "-") == 0){
		    if(reverse){
			 (void)fprintf(stderr,
				       "Can not use reverse flag with stdin\n");
			 exit(1);
		    }
		    else
			 dumpfile = stdin;
	       }
	       else if(!(dumpfile = fopen(optarg, "r"))){
		    (void)perror(optarg);
		    errflag++;
	       }
	       break;
	  case 'o':
	       if(!errflag && !(outfile = fopen(optarg, "w"))){
		    (void)perror(optarg);
		    errflag++;
	       }
	       break;
	  case 'h':
	       helpscreen(argv[0]);
	       exit(0);
	       break;
	  case 'U':
	       unbuffered = 1;
	       break;
	  default:
	       errflag++;
	       break;
	  }
     }

     defargs.argp = sparformat;
     
     if(errflag)
	  exit(1);

     if(!file && !expr)
	  (void)addmembuf("{print}", "");

     if(!dumpfile)
	  if(!(dumpfile =fopen(DEFAULT_PACCT, "r"))){
	       (void)perror(DEFAULT_PACCT);
	       exit(1);
	  }

     setlogfile(outfile);
     setlogmode(logmode);
     setclockrate();
     if((parsetree = parse())){
	  extract(fileno(dumpfile), parsetree, reverse);
	  flushoutput(logfile);
     }
     else
	  exit(1);
     return 0;
}

void
helpscreen(char *name)
{
     (void)fprintf(stdout,"\
%s: usage: %s [options]\n\
	options: -a		ASCII output (the default)\n\
		 -b		Binary output\n\
                 -r             Print records in reverse order\n\
                 -U             Don't do buffered I/O\n\
		 -h		Print this help screen\n\
		 -e script	Specify script inline\n\
		 -E script	Specify script inline\n\
		 -f file	Specify script filename\n\
		 -F file	Specify script filename\n\
		 -d acctfile	Specify pacct file to process\n\
		 -o outfile	Specify output file (def: stdout)\n\
\n\
%s processes UNIX process accounting files, selecting records to\n\
output.  The syntax of the script file is similar to awk(1).\n\
The script can be specified on the command line or can be stored\n\
in a file.  All scripts specified by '-e', '-E', '-f', '-F' are\n\
in effect, processed as a single file.  Scripts specified by '-e' and\n\
'-f' are appended, whereas scripts specified by '-E' and '-F' are pre-\n\
pended.  The use of '-E' or '-F' along with a '#!%s -f' script\n\
executable allows insertion of additional script lines from the command\n\
line.\n",
	     name, name, name, name);
}

static int
nread(int fd, char *buf, int numbytes)
{
     int count = numbytes;
     int n;

     while(count){
	  if((n = read(fd, buf, count)) > 0){
	       buf += n;
	       count -= n;
	  }
	  else
	       break;
     }
     return numbytes-count;
}

static void
extract(int fd, struct parsetree *pt, int revflag)
{
     struct acct *buf;
     int i, n;
     int bsize = sizeof(struct acct)*1024;

     if(unbuffered)
	  bsize=sizeof(struct acct);

     buf = (struct acct *)malloc(bsize);

     if(!revflag){
	  while((n=nread(fd, (char *)buf, bsize)) > 0){
	       for(i=0;i<(n/sizeof(struct acct));i++)
		    interp(&buf[i], pt);
	  }
     }
     else {
	  int prev;
	  prev = lseek(fd, -bsize , SEEK_END);
	  while((n=nread(fd, (char *)buf, bsize)) > 0){
	       for(i=(n/sizeof(struct acct))-1;i>=0;i--)
		    interp(&buf[i], pt);
	       if(!prev)
		    break;
	       if(prev < bsize){
		    (void)lseek(fd, 0, SEEK_SET);
		    bsize = prev;
		    prev = 0;
	       }
	       else {
		    prev = lseek(fd, -(bsize<<1), SEEK_CUR);
	       }
	  }
     }
     (void)flushbuf();
     free(buf);
}


#define OUTBUFSIZE 1000
struct acct *outbuffer = 0;
static int outptr = 0;

void
writebuf(struct acct *outbuf, struct args *al)
{
     switch(outlogmode){
     case LOGBINARY:
	  writebin(outbuf, al);
	  break;
     case LOGASCII:
	  writeascii(outbuf, al);
	  break;
     default:
	  (void)fprintf(stderr, "Internal error... unknown output mode.\n");
	  exit(1);
	  break;
     }
}

static void
writebin(struct acct *outbuf, struct args *al)
{
     if(!outbuffer)
	  outbuffer = (struct acct *)malloc(sizeof(struct acct)*OUTBUFSIZE);
     if(outptr == OUTBUFSIZE)
	  (void)flushbuf();
     (void)memcpy((void *)&outbuffer[outptr++],
		  (void *)outbuf,
		  sizeof(struct acct));
     if(unbuffered)
	  (void)flushbuf();
}

static void
flushbuf(void)
{
     if(outptr)
	  (void)write(fileno(logfile), (char *)outbuffer,
		sizeof(struct acct)*outptr);
     outptr = 0;
}

static void
writeascii(struct acct *outbuf, struct args *al)
{

     if(al){
	  writeformatted(logfile, outbuf, al);
     }
     else {
	  writeformatted(logfile, outbuf, &defargs);
     }
}

